/*
 * Copyright (c) 2022-2023 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */
#include <stdlib.h>

#include <doca_log.h>
#include <doca_argp.h>

DOCA_LOG_REGISTER(FLEXIO_RPC::MAIN);

/* Sample's logic */
int flexio_rpc(int argc, char **argv);

/*
 * Sample main function
 *
 * @argc [in]: command line arguments size
 * @argv [in]: array of command line arguments
 * @return: EXIT_SUCCESS on success and EXIT_FAILURE otherwise
 */
int
main(int argc, char *argv[])
{
	doca_error_t result;
	int exit_status = EXIT_SUCCESS;
	struct doca_logger_backend *stdout_logger = NULL;

	/* Create a logger backend that prints to the standard output */
	result = doca_log_create_file_backend(stdout, &stdout_logger);
	if (result != DOCA_SUCCESS)
		return EXIT_FAILURE;

	result = doca_argp_init("flexio_rpc", NULL);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to init ARGP resources: %s", doca_get_error_string(result));
		return EXIT_FAILURE;
	}
	result = doca_argp_start(argc, argv);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("Failed to parse sample input: %s", doca_get_error_string(result));
		doca_argp_destroy();
		return EXIT_FAILURE;
	}

	/* Run sample */
	result = flexio_rpc(argc, argv);
	if (result != DOCA_SUCCESS) {
		DOCA_LOG_ERR("flexio_rpc() encountered errors");
		exit_status = EXIT_FAILURE;
	}

	/* ARGP cleanup */
	doca_argp_destroy();
	return exit_status;
}
